######################################################################
#:Clearcase Class
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2009. All Rights Reserved.
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

=head1 NAME

CQCC::CCase - Base class to manage communications with ClearCase

=head1 CLASS DESCRIPTION

This class provides a base class for all ClearCase interfaces and also
manages a standard interface to ClearCase using external cleartool
processes. The CCase class is used by default; to select a different
subclass the configuration file must ask for it as the
ClearCaseInterface class as shown above in the TriggerConfig.pl file.

&SetConfigParm("ClearCaseInterface", "CQCC::CCase");

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=head1 METHODS

=cut

######################################################################
#::Package CQCC::CCase
######################################################################

package CQCC::CCase;
use strict;
sub Version { return '2.1.1' };

use CQCC::IOBase;     # Include in case we need to make own copy
use CQCC::Parm;       # Implicitly included in iobase
use CQCC::AssocList;  
use CQCC::Timer; 
use Config;

######################################################################
#::Class Globals 
######################################################################

my %VOCache;          # Used to cache requests for Vob/OID information
my $outFile;          # Output file for cleartool output
my $windows = ($Config::Config{osname} eq "MSWin32") ? 1 : 0;
my $sep = $windows ? "\\" : "/";
my $op;               # operation type

######################################################################
#::function ClassInit

=over 6

=item C<ClassInit()>

This class function is automatically invoked to define class 
parameters using CQCC::Parm objects. Current parameters include: 

=over 4

=item C<CLEARCASE_OP_KIND>

ClearCase environment variable for the requested operation type. 

=item C<CLEARCASE_PN>

ClearCase environment variable for the unextended pathname.

=item C<CLEARCASE_XPN>

ClearCase environment variable for the extended pathname.

=item C<CLEARCASE_VOB_PN>

ClearCase environment variable for the VOB pathname.

=item C<ClearCaseInterface>

Name of CCase subclass to use for this session, used by
IOClassFactory(). Set to "CQCC::CCase" by default.

=back

=back

=cut

######################################################################

sub ClassInit { 
    if (&ParmDefined("CLEARCASE_PN")) {
        return;
    }     

    &SetParmDefaults(PCLASS=>"CQCC::CCase", SRC_EV=>1);

    &DefineParm(NAME=>"CLEARCASE_PN");
    &DefineParm(NAME=>"CLEARCASE_XPN");
    &DefineParm(NAME=>"CLEARCASE_VOB_PN");

    &DefineParm(NAME=>"CLEARCASE_SERIES_ID");
    &DefineParm(NAME=>"CLEARCASE_BEGIN_SERIES");
    &DefineParm(NAME=>"CLEARCASE_END_SERIES");

    &DefineParm(NAME=>"CLEARCASE_OP_KIND");
    &DefineParm(NAME=>"CLEARCASE_PPID");
    &DefineParm(NAME=>"CLEARCASE_COMMENT");

    &SetParmDefaults(PCLASS=>"CQCC::CCase", SRC_CFG=>1);
    &DefineParm(NAME=>"ClearCaseInterface", DEFAULT=>"CQCC::CCase",
                SRC_CMD=>1);

    &SetParmDefaults(PCLASS=>"CQCC::CCase");
    &DefineParm(NAME=>"CACHE_CHECKOUT_NOT_SELECTED",
                SRC_INT=>1, SRC_CACHE=>1, DEFAULT=>0);

    $op = &GetParm("CLEARCASE_OP_KIND");
    &SetParmDefaults();
}

&ClassInit();

######################################################################
#::function ClassTest

=over 6

=item C<ClassTest(@args)>

This class function uses sets up a local environment for testing;
calls IOClassFactory() to get an instance of the desired subclass; 
and invokes the object's SelfTest method for testing. 

=back

=cut

######################################################################

sub ClassTest { 
    my @args = @_;
    my @requiredCCEVs = ("PN", "VOB_PN", "XPN", "OP_KIND");
    my @missingEVs;

    for (@requiredCCEVs) { 
        if (! $ENV{"CLEARCASE_$_"} ) { 
            push @missingEVs, "CLEARCASE_$_";
        } 
    } 
    if ($#missingEVs > -1) { 
        print "CCase::ClassTest requires some missing " . 
                "Environment Vars:\n" . 
            join(", ", @missingEVs), "\n";
        &IOExitHandler(2);
    }
    my $ccase = &IOClassFactory("ClearCaseInterface");
    $ccase->SelfTest(@args);
}
    
######################################################################
#::method EditLinks

=over 6

=item C<EditLinks($AddAssocList, $DeleteAssocList)>

This method changes the current associations being maintained in 
ClearCase by adding the items in the AddAssocList and deleting those
in the DeleteAssocList. It will complain about preexisting links
that may be overridden but it will not take special action on them. 

=back

=cut

######################################################################

sub EditLinks {
    my ($self, $addList, $delList) = @_;

    my %linkKeys = $self->LinkKeys();
    # Assoc Keys are in the format "entity:database000000n"
    my @addAssocs = (defined $addList) ? $addList->GetAssocs() : ();
    my @delAssocs = (defined $delList) ? $delList->GetAssocs() : ();
    my $key;
    my $result = 1;

    for $key (@addAssocs) {
        my ($entity,$bugname) = split(':',$key);

        # Add links
        # Case: the hyperlink already exists on this vob object
        if ($linkKeys{$key}) { 
            &IODebugPrint("DETAILED",
                "found pre-existing hlink for $key\n");
        } else {
            my $ccname;
            if ((&GetParm("CQCC_MKHLINK_CCXPN") eq "TRUE") && $windows) {
                 $ccname = "\"" . &GetParm("CLEARCASE_XPN");
                 substr($ccname,rindex($ccname,'\\',length($ccname)-1))='\\CHECKEDOUT"';
                 # If the same file is checked out multiple times in the
                 # same view, the mkhlink command may be issued against
                 # the wrong version.  The dbid is added at the end and
                 # at this point we cannot determine if this is the case.
                 &IODebugPrint("DETAILED",
                      "Using extended pathname for mkhlink $ccname\n");
            } else {
                 $ccname = $self->GetCCname();
            }
            my $cmd = "mkhlink -unidir -ttext $bugname -ftext " .
                      "$entity -nc $self->{HLINKTYPE} $ccname";
            my @results = $self->PostCommand($cmd);

        #Returns 'Created hyperlink "CrmRequest@nnn@vob"'
            my $found = 0;
            my $line;
            foreach $line (@results) {
                if ($line =~  /\"($self->{HLINKTYPE}\@\d+\@\S+)\"/) {
                    $linkKeys{$key} = $1;
                    $found = 1;
                    last;
                }
            }
            if ($found == 0) {
                my $errMsg = "\ncleartool $cmd\n\n";
                $errMsg .= join("\n",@results) . "\n\n";
                my @info =
                    $self->PostCommand("describe " .
                         " hltype:$self->{HLINKTYPE}");
                $errMsg .= join("\n",@info) . "\n";

                &IOMessage("ERROR", [ "CC_E_NO_HLINK", $errMsg ]);
                $result = 0;
            }
        }
    }

    for $key (@delAssocs) {
        my ($entity,$bugname) = split(':',$key);

        # Delete the link 
        if ($linkKeys{$key}) {
            $self->PostCommand("rmhlink -nc $linkKeys{$key}");
            delete($linkKeys{$key});
        }
    }

    # Resave the modified hash for the link keys
    $self->{LINK_KEYS} = \%linkKeys;
    return $result;
}

######################################################################
#::method ExistingAssocList

=over 6

=item C<ExistingAssocList()> 

This method gets the current association LinkKeys hash list and 
converts it to an AssocList for return. 

=back

=cut

######################################################################

sub ExistingAssocList {
    my ($self) = @_;

    my %links = $self->LinkKeys();
    my @assocs = (sort (keys %links));
    return (new CQCC::AssocList(ASSOCS=>\@assocs));
}

######################################################################
#::method GetCCname

=over 6

=item C<GetCCname()>

This method returns the version pathname that should be used in 
Clearcase operations to get more information about the version.

=back

=cut 

######################################################################

sub GetCCname {
    my ($self) = @_;
    if ($self->{PN_MODE} eq "EDIT") {
        return &IOQuotePathname($self->{XPN});
    }

    # For post-checkout, pre-uncheckout or pre-checkin use the oid
    # Get the OID the first time only to minimize overhead
    # by caching the oid/pathname combination for later re-use

    my $tr_op = $ENV{"CLEARCASE_TRTYPE_KIND"};
    my $pre_op = ($tr_op eq "pre-operation") ? 1 : 0;

    if (($op eq "checkout" && ! $pre_op) || 
        ($op eq "uncheckout" && $pre_op) ||
        ($op eq "checkin" && $pre_op)) {

        my ($oid,$oidPath) = split("\n",$self->{CCNAME_CACHE});
        if (! $oid || $oidPath ne $self->{PN}) {
            my $ccname =  &IOQuotePathname($self->{PN});
            my @results = $self->PostCommand(
                "lsco -cview -fmt \"%On\" -d  $ccname ");
            my $line = $results[0];
            chomp $line;
            # Valid result can only contain hex digits, colon, and period
            if ($line =~ /^([A-Fa-f0-9:.]*)/) {
                $oid = "$1\@" . $self->{VOB_PN};
                $self->{CCNAME_CACHE} = "$oid\n$self->{PN}";
            } else {
                &IODebugPrint("BASIC", 
                    "CCase::GetCCname: Error getting OID using lsco\n" . 
                    @results);
                &IOMessage("FATAL", ["CC_E_VOLIST", $self->{PN} . 
                            "\n" . join("\n",@results) . "\n"]);
            }
        }
        return "oid:$oid";
    }
    # Don't expect to get here but cases outside post-checkout to pre-checkin
    # are only safe cases where XPN shouldn't be used. 
    &IODebugPrint("BASIC", "CCase::GetCCname not EDIT mode or expected cases\n");
    return &IOQuotePathname($self->{XPN});
}

######################################################################
#::method GetVOList

=over 6

=item C<GetVOList( "PN" or "XPN")>

This method is used to retrieve a list containing the pathname (PN) or
extended pathname (XPN), VOB uuid, and object ID for the current
version; a fourth element includes the old OID (checkin needs to know
both checkout and checkin OIDs). When used in a scalar context it
returns a string with a comma separated list instead. 

($cqname, $uuid, $oid, $oldOID) = $cc->GetVOList(pathname);

=back

=cut 

######################################################################

sub GetVOList {
    my ($self, $nameTag) = @_;
    my $VOstr;
    my ($uuid, $oid, $dynamic_oid, $oldOID, @voList);
    my ($slash, $slash1, $nameholder);
    my $derived = 0;

    # name as seen in ClearQuest - extended name for checkin only

    my $CQname = ($nameTag eq "PN") ? $self->{PN} : $self->{XPN};
    my $name = $self->{PN};

    # Do CT dump, once for the vob oid and again for the version oid
    # Send stderr to NULL device because you can get an spurious
    # error message when you provide a view extended pathname when 
    # you don't have a view set.

    if ($VOCache{$CQname}) {
        @voList = @{ $VOCache{$CQname} }; 

    } else {
        my $ccname = ($self->{PN_MODE} eq "EDIT") ? &IOQuotePathname($CQname) :
                                                    &IOQuotePathname($name);
dumpcommand:
        my @results = 
            $self->PostCommand("dump vob:$self->{VOB_PN} $ccname");

        for (@results) {
            my $line = $_;
        # For Derived Objects, the dump command retrieves the data
        # for the previous version.  This cause a problem during
        # checkin when we try to match the OIDs with the data CQ
        # has stored.  To get the correct data, we must run the
        # dump command asking for the checkedout version.  We
        # will do this be adding CHECKEDOUT after the last
        # seperator.
            if ($derived < 1) {
               if ((/mtype=derived/) && ($op eq "checkin")) {
                   $ccname = &GetParm("CLEARCASE_XPN");
                   $derived = 1;
                   $slash = rindex($ccname, $sep);
                   if ($slash == -1) {
                       # If for some reason we could not format the filename
                       # to run the dump command, just let it fall thru and
                       # and we will fail down below when the OIDs do not
                       # match up.
                       $derived = 0;
                   } else {
                       $ccname = substr($ccname, 0, $slash);
                       $ccname = &IOQuotePathname($ccname . $sep . "CHECKEDOUT");
                       $uuid = 0;
                       $oid = 0;
                       goto dumpcommand;
                   }
               }
            }

            if (/oid=(\S*)/) {
                if (!$uuid) {
                    $uuid = $1;
                } elsif (!$oid) {
                    $oid = $1;
                }
            } elsif (/\S+\s\(([A-Fa-f0-9:.]*)\)/) {
                if (!$dynamic_oid && $line !~ /view=/ && 
                    length($1) >= 40) { 
                    $dynamic_oid = $1;
                }
            }
        }

        $oldOID = $oid;
        my $ccOIDname = $self->GetCCname();
        &ForceParm("CACHE_CHECKOUT_NOT_SELECTED",0);
        if ($ccOIDname =~ /oid:([A-Fa-f0-9:.]*)\@/) {
            my $checkedoutOID = $1;
            if ($oid ne $checkedoutOID) {
                if ((GetParm("CLEARCASE_OP_KIND") eq "checkin") && 
                    (GetParm("CQCC_POSTCHECKIN_COMMIT") ne "TRUE")) {
                    &IOMessage("FATAL",["CC_E_CONFIG_SPEC_CONFLICT"]);                
                }
                &ForceParm("CACHE_CHECKOUT_NOT_SELECTED",1);
                # Correct oid information for checkouts
                $oid = $checkedoutOID;
            }
        }

        if ($nameTag eq "XPN" && 
              $ENV{CLEARCASE_VIEW_KIND} eq "dynamic"
              && $dynamic_oid) {
            $oid = $dynamic_oid;
        }

        if (!$uuid || !$oid) {
            IOMessage("FATAL", ["CC_E_VOLIST", $name]);
            IOExitHandler(1);
        }
        else {
           my $CQpathname = $CQname;
           # If inside CCRC, trim off the backing-store view name
           # for usability (to shorten pathname) and
           # for reliability (storage path can change between co and ci)

           # On UNIX use /vob-tag/pathname like dynamic views
           # On Windows use M:/ccweb/vob-tag/pathname

           if ($ENV{CCRC_STORAGE_DIR}) {
               my $storage=$ENV{CCRC_STORAGE_DIR};
               my $vobName = &GetParm("CLEARCASE_VOB_PN");
               # Escape out the \'s for pattern match
               if ($windows) {
                   $storage =~ s/\\/\\\\/g;  
                   $vobName =~ s/\\/\\\\/g;  
               }
              
               if ($CQpathname =~ /$storage[\s\S]+?($vobName[\s\S]*)/) {
                   $CQpathname = $windows ? "M:\\ccweb$1" : $1;
               }
           }
           @voList = ($CQpathname, $uuid, $oid, $oldOID);
        }
        $VOCache{$CQname} = \@voList;

        &IODebugPrint("BASIC","GetVOList: $ccname\n" . 
                      "    CCname: $ccOIDname\n" . 
                      "    XPN: $self->{XPN}\n" . 
                      "    CQname: $CQname\n    VOBID: $uuid\n" . 
                      "    OID: $oid\n    Old OID: $oldOID\n");
    }

    
    if (wantarray()) { 
        return @voList;
    } 
    return join(",", @voList);
}

######################################################################
#::method LinkKeys

=over 6

=item C<LinkKeys()>

Internal method used to retrieve the existing hyperlinks from
ClearCase for the current version being worked on. The returned hash
list associates keys (CQRequests in the form <entity>:<db><00000nnn>,
i.e. "defect:SAMPL00000008") with ClearCase CrmRequest hyperlinks
(<linktype>@<id>@<vobtag>, i.e. "CrmRequest@0700@\cqcc_test".

%linkKeysList = $cc->LinkKeys()>

Once built, it is stored locally in an object instance variable for
later reference and will be modified by EditLink() operations in
parallel with commands to ClearCase to change the associations to
avoid the need to reload the array each time a change is made.

Note: The Entity name should be expected to vary with different links
so derive per link as needed. 

=back

=cut

######################################################################

sub LinkKeys {
    my $self = shift;

    if (! defined $self->{LINK_KEYS}) {
        my %linkKeys = ();
        my $ccname = $self->GetCCname();

        my @dump = $self->PostCommand(
            "desc -long -ahlink $self->{HLINKTYPE} $ccname");

        for (@dump) {
           # <ws><text><w><quoted-text><->><ws><quoted-text>
           if (/^\s+(\S+)\s+"(\S+)"\s+->\s+"(\S*)"/) {
               my $key = $2 . ":" . $3;
               # concat entitydef & bugID, save the hlink's ID
               $linkKeys{$key} = $1;
               &IODebugPrint("DETAILED", 
                   "LinkKeys: hlink found: $key -> $3\n");
           }
        }
        $self->{LINK_KEYS} = \%linkKeys;
    }

    return %{ $self->{LINK_KEYS} };
}

######################################################################
#::method new

=over 6

=item C<new CQCC::CCase()>

This method presets the objects attributes to read in the basic
ClearCase trigger environment values (PN, XPN, VOB_PN); defines the
hyperlink type name and ClearTool process pathname; and presets the
data list to empty. 

=back

=cut

######################################################################

sub new { 
   my $proto = shift;
   my $class = ref($proto) || $proto;  # Object or class name

   # Attributes
   my $self = {
       CLASSNAME => "CQCC::CCase",
       HLINKTYPE => 'CrmRequest',  # Hyperlink name vs attribute
       PN =>                       # Pathname
           &GetParm("CLEARCASE_PN"),
       XPN =>                      # Extended pathname
           &GetParm("CLEARCASE_XPN"),

       VOB_PN =>                   # VOB pathname
           &GetParm("CLEARCASE_VOB_PN"),
       CCNAME_CACHE => undef,      # GetCCname results cache
       PN_MODE => "CCASE",         # CCASE (trigger based) or 
                                   # EDIT (command line invocation)
       LINK_KEYS => undef,         # Links hash
       CLEARTOOL => "cleartool",   # path to cleartool executable
                                   # Expects atria in path
       USE_PIPELINE => "FALSE",     # Enable/disable use of pipeline
       @_,                         # Override previous attributes
       @_,                         # Extra time for ccperl
   };

   $outFile = &IOPathname("TEMP", "cqcc_ccase.out");
   bless($self, $class);
   return $self;
}

######################################################################
#::method PostCommand

=over 6

=item C<PostCommand($operation)>

This method sends a ClearCase command to ClearTool and 
returns an array of output lines. 

@resultLines = $cc->PostCommand($operation);

=back

=cut

######################################################################

sub PostCommand {
    my ($self,$operation) = @_;
    my $t1 = &TimerStart("CCase::PostCommand $operation");
    my $cmd = $self->{CLEARTOOL} . " " . $operation;
    my ($package, $filename, $line) = caller;
    my @result;

    &IODebugPrint("DETAILED",
         "ClearCase PostCommand: ($filename)\n   $cmd\n");

    if ($self->{USE_PIPELINE} eq "TRUE") {
        @result = $self->_PostCommandPiped($operation);
    } else {
        my $ret = system "$cmd >$outFile 2>&1";
        @result = &IOFileRead($outFile,1);
    }
    &IODebugPrint("DETAILED", "PostCommand: " . 
        join("\n", @result) . "\n");
    &TimerStop($t1);
    return @result;
}

######################################################################
#::method _PostCommandPiped

=over 6

=item C<_PostCommandPiped($cmd)>

This internal method manages the cleartool connection using a two
way pipeline and is used when the USE_PIPE attribute is enabled. 

@resultLines = $cc->_PostCommandPiped($cmd);

=back

=cut

######################################################################

my $spawnedCT;

sub _PostCommandPiped { 
    my ($self, $cmd) = @_;

    # maintains global file handles CT_WRITE and CT_READ

    local($_);  #??
    my $cmdTerminator = ('ThisIsABadCommand');
    my $ct = $self->{CLEARTOOL};
    my (@response);
    
    # Spawn a cleartool process if we haven't yet done so
    if (! $spawnedCT) {

        # Open pair of connected \pipelines
        pipe(CT_READ, CT_P_WRITE) || die("can't create pipe\n");

        my $oldfh = select(CT_P_WRITE); 
        $| = 1; 
        select($oldfh);

        # redirect STDOUT to CT_P_WRITE, ditto for STDERR

        open(SAVED_STDOUT, ">&STDOUT") || die("can't dup stdout\n");
        close(STDOUT);
        open(STDOUT, ">&CT_P_WRITE") || 
            die("can't redirect STDERR\n");

        open(SAVED_STDERR, ">&STDERR") || die("can't dup stderr\n");
        close(STDERR);
        open(STDERR, ">&CT_P_WRITE") || 
            die("can't redirect STDERR\n");

        # spawn the cleartool and get a WRITE fh for its input.
        # Its output / errout is inherited from our (redirected) 
        #    STD{OUT,ERR}

        open(CT_WRITE, "| $ct") || die("can't spawn $ct\n");
        $oldfh = select(CT_WRITE); $| = 1; select($oldfh);

        close(CT_P_WRITE);   # this isn't our side of the pipe

        # redirect STD{OUT,ERR} back to their original state
        close(STDOUT);
        open(STDOUT, ">&SAVED_STDOUT");
        close(SAVED_STDOUT);

        close(STDERR);
        open(STDERR, ">&SAVED_STDERR");
        close(SAVED_STDERR);

        $spawnedCT = "yes";
    }

    # Send the command.
    print CT_WRITE  $cmd, "\n";

    # Send a bad command so we can recognize the end of the response
    print CT_WRITE  "$cmdTerminator", "\n";

    # Get the response
    while (<CT_READ>) {
        chomp;
        last if (/$cmdTerminator/);
        push(@response, $_);
    }
    return @response;
}

######################################################################
#::method SelfTest

=over 6

=item C<SelfTest(@args)>

This method runs basic unit testing on the object instance which 
currently includes creating hyperlinks; displaying the current 
hyperlinks to show that they exist; deleting the new hyperlinks;
and then displaying the current hyperlinks to show confirm deletion.

=back

=cut

######################################################################

sub SelfTest {
    my ($self, @args) = @_;

    ##############################
    print "Should get cleartool: Error: Unrecognized command: foo\n";
    my @result = $self->PostCommand("foo");
    print join("\n",@result), "\n";

    @result = $self->PostCommand("help ls");
    print join("\n",@result), "\n";
    
    ##############################
    my ($uuid, $oid, $cqname) = $self->GetVOList("XPN");
    print "UUID: $uuid  \n   OID: $oid  \n   NAME: $cqname \n";

    ##############################
    print "BEFORE new link made-----------\n";
    my %links = $self->LinkKeys();
    my $lkey;
    for $lkey (keys %links) {
        print "link: $lkey => $links{$lkey}\n";
    }

    my $db = &ParmDefined("CQCC_DATABASE") ? 
       &GetParm("CQCC_DATABASE") : "SAMPL";
    print "Should see two new links for " . $db . "155, " . 
       $db . "160\n";
    my $assocList = new CQCC::AssocList();
    $assocList->ProcessBugString("defect",$db . "155 160");
    
    $self->EditLinks($assocList, undef);
    print "AFTER new links made-----------\n";
    %links = $self->LinkKeys();
    for $lkey (keys %links) {
        print "link: $lkey => $links{$lkey}\n";
    }

    $self->EditLinks(undef, $assocList);
    print "AFTER new link deleted-----------\n";
    %links = $self->LinkKeys();
    for $lkey (keys %links) {
        print "link: $lkey => $links{$lkey}\n";
    }
    return 0;
}

######################################################################
#::method SetXPN

=over 6

=item C<SetXPN(vob_pn, xpn)>

This method overrides the environment settings for the VOB_PN and
XPN values from the provided arguments. It also adjusts the PN
value as well. It is intended only for use by command line operations
that are not being invoked through the ClearCase environment. 

=back

=cut

######################################################################

sub SetXPN {
    my ($self, $vob_pn, $xpn) = @_;
    $self->{VOB_PN} = $vob_pn;    
    $ENV{"CLEARCASE_VOB_PN"} = $vob_pn;
    &ForceParm("CLEARCASE_VOB_PN", $vob_pn);

    $self->{XPN} = $xpn;    
    $ENV{"CLEARCASE_XPN"} = $xpn;
    &ForceParm("CLEARCASE_XPN", $xpn);

    ($self->{PN}) = split("@@",$xpn);
    $ENV{"CLEARCASE_PN"} = $self->{PN};
    &ForceParm("CLEARCASE_PN", $self->{PN});

    $self->{PN_MODE} = "EDIT";
}

######################################################################
#::method UnCheckout

=over 6

=item C<UnCheckout($pathname)>

This method unchecks out the provided pathname. This is used when
a postop checkout operation fails as a way to cancel the checkout
operation. 

=back

=cut 

######################################################################

sub UnCheckout {
    my ($self, $pn) = @_;
    my $ccname = &IOQuotePathname($pn);
    # Note: Can't use extended or oid-based pathnames here, 
    # element name should be fine
    $self->PostCommand("unco -rm $ccname");
}

######################################################################
#::Tail
######################################################################

1;
